package ninja.thefirearchmage.games.fourtykcalculator;

import lombok.Getter;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

public class Weapon {
    @Getter
    private String name;
    @Getter
    private int strength;
    @Getter
    private int ap;
    @Getter
    private int attacks;
    @Getter
    private WeaponDamage damage;
    @Getter
    private WeaponRangeType rangeType;
    @Getter
    private int hitValue;
    private boolean hasLethalHits;
    @Getter
    private Optional<Integer> sustainedHits;
    private boolean hasDevastatingWounds;
    private boolean hasHazardous;
    private Map<UnitTypeKeyword, Integer> antiStats;
    private boolean hasHeavy;
    private boolean hasBlast;
    private boolean hasLance;
    private boolean ignoresCover;
    private boolean hasIndirectFire;
    @Getter
    private Optional<Integer> melta;
    @Getter
    private Optional<Integer> rapidFire;

    public Weapon() {
        this.antiStats = new HashMap<>();
        this.melta = Optional.empty();
        this.rapidFire = Optional.empty();
        this.sustainedHits = Optional.empty();
    }

    public boolean hasLethalHits() {
        return hasLethalHits;
    }

    public boolean hasDevastatingWounds() {
        return hasDevastatingWounds;
    }

    public boolean hasHazardous() {
        return hasHazardous;
    }

    public boolean isHeavy() {
        return hasHeavy;
    }

    public boolean hasBlast() {
        return hasBlast;
    }

    public boolean ignoresCover() {
        return ignoresCover;
    }

    public boolean hasLance() {
        return hasLance;
    }
    public boolean hasIndirectFire() {
        return hasIndirectFire;
    }

    /**
     * Returns whether this weapon has Anti-X where X is the given unit type.
     */
    public Optional<Integer> getAnti(UnitTypeKeyword unitType) {
        return Optional.ofNullable(antiStats.get(unitType));
    }

    /**
     * Returns whether this weapon is anti for any of the given types
     */
    public boolean isAntiFor(Set<UnitTypeKeyword> unitTypes) {
        var antiUnitTypes = antiStats.keySet();
        return unitTypes.stream().anyMatch(antiUnitTypes::contains);
    }
    public Optional<Integer> getBestAntiFor(DefensiveProfile target) {
        var bestAnti = Optional.<Integer>empty();
        if(isAntiFor(target.getUnitTypeKeywords())) {
            var antiUnitTypes = antiStats.keySet();
            var minAnti = target.getUnitTypeKeywords().stream()
                    .filter(antiUnitTypes::contains)
                    .mapToInt(type -> antiStats.get(type))
                    .min().getAsInt();
            bestAnti = Optional.of(minAnti);
        }

        return bestAnti;
    }
}
