package ninja.thefirearchmage.games.fourtykcalculator.utils.datastructures;

import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;

public class Either<L,R> {
    private final Optional<L> leftResult;
    private final Optional<R> rightResult;

    private Either(Optional<L> leftResult, Optional<R> rightResult) {
        this.leftResult = leftResult;
        this.rightResult = rightResult;
    }

    public static <L, R> Either<L, R> right(R value) {
        return new Either<>(Optional.empty(), Optional.of(value));
    }

    public static <L, R> Either<L, R> left(L value) {
        return new Either<>(Optional.of(value), Optional.empty());
    }

    public boolean isLeft() {
        return this.leftResult.isPresent();
    }

    public boolean isRight() {
        return this.rightResult.isPresent();
    }

    public R getRight() {
        return this.rightResult.orElseThrow(()-> new IllegalStateException("This either is Left"));
    }
    public R get() {
        return getRight();
    }
    public <U> Either<L, U> map(Function<R,U> mapperFunction) {
        return new Either<>(this.leftResult, this.rightResult.map(mapperFunction));
    }

    public L getLeft() {
        return this.leftResult.orElseThrow(() -> new IllegalStateException("This either is Right"));
    }

    public void ifRight(Consumer<R> function) {
        rightResult.ifPresent(function);
    }

    public void ifLeft(Consumer<L> function) {
        leftResult.ifPresent(function);
    }
}
